<!DOCTYPE html>
<html lang="en">
<head>
  <title>simpleMediaCenter</title>
  <meta charset="UTF-8" />
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <meta name="description" content="simpleMediaCenter main page" />

  <link xmlns="http://www.w3.org/1999/xhtml" rel="stylesheet" type="text/css" href="reset.css" />
  <link rel="stylesheet" href="//netdna.bootstrapcdn.com/bootstrap/3.1.1/css/bootstrap.min.css">
  <link xmlns="http://www.w3.org/1999/xhtml" rel="stylesheet" type="text/css" href="bootstrap.spacelab.min.css" />
  <link href="//netdna.bootstrapcdn.com/bootstrap/3.0.0/css/bootstrap-glyphicons.css" rel="stylesheet">
  <link xmlns="http://www.w3.org/1999/xhtml" rel="stylesheet" type="text/css" href="style.css" />
</head>
<body>
<div class="navbar navbar-default navbar-fixed-top active" role="navigation">
    <div class="container">
        <div class="navbar-header">
            <a href="./" class="navbar-brand">simpleMediaCenter</a>
        </div>
      {% include "playernav.html" %}
    </div>
</div>

<div class="container">
    <div class="row">
      {% if displaySidebar %}
        <div class="col-sm-9">
      {% else %}
        <div class="col-sm-12">
      {% endif %}
            {% include "alertbox.html" %}
            <section id=content>
                {% block content %}
                {% endblock %}
            </section>
        </div>
      {% if displaySidebar %}
        {% include "sidebar.html" %}
      {% endif %}
    </div>
</div>

<!--javascript-->
<script src="//code.jquery.com/jquery.js"></script>
<script src="//netdna.bootstrapcdn.com/bootstrap/3.1.1/js/bootstrap.min.js"></script>
<script src="./simpleMediaCenter.js"></script>
</body>
</html>
