{% if playlistArray %}
<div id="playlist" class="panel panel-success">
    <div class="panel-heading">
        <div class="dropdown">
            <a class="dropbown-toggle panel-title" data-toggle="dropdown" href="#">Playlist - {{activePlaylist}}
                <span class="caret"></span>
            </a>
            <ul class="dropdown-menu">
              {% for playlist in playlistArray%}
                <li>
                    <a href="./selectPlaylist?id={{loop.index0}}">{{playlist}}</a>
                </li>
               {% endfor %}
            </ul>
        </div>
    </div>
    <div class="panel-body">
        <div class="list-group">
          {% for entry in playlistFiles %}
            <a href="#"  class="list-group-item">{{entry}}</a>
          {% endfor %}
        </div>
    </div>
</div>
{% endif %}