<a href="./stop" id="btn-stop" class="btn btn-default navbar-btn playerbutton">
    <span class="glyphicon glyphicon-stop"></span>
</a>
<a href="./pause" id="btn-resume" class="btn btn-default navbar-btn playerbutton">
    <span class="glyphicon glyphicon-play"></span>
</a>
<a href="./pause" id="btn-pause" class="btn btn-default navbar-btn playerbutton">
    <span class="glyphicon glyphicon-pause"></span>
</a>
<a href="./volumedown" id="btn-volume-down" class="btn btn-default navbar-btn playerbutton volumecontrol">
    <span class="glyphicon glyphicon-volume-down"></span>
</a>
<a href="./volumeup" id="btn-volume-up" class="btn btn-default navbar-btn playerbutton volumecontrol">
    <span class="glyphicon glyphicon-volume-up"></span>
</a>
