{#

                <button type="button" class="close">×</button>

    <div class="modal-content">
      <div class="modal-header">
        <button type="button" class="close" data-dismiss="modal" aria-hidden="true">×</button>
        <h4 class="modal-title">Modal title</h4>
      </div>
      <div class="modal-body">
        <p>One fine body…</p>
      </div>
      <div class="modal-footer">
        <button type="button" class="btn btn-default" data-dismiss="modal">Close</button>
        <button type="button" class="btn btn-primary">Save changes</button>
      </div>
    </div>
    
    <button type="button" onclick='$("#exception").show();' >show modal</button>
<button type="button" onclick='$("#exception").hide();' >hide modal</button>
#}

<div class="modal" id="exception">
  <div class="modal-dialog">
        <div class="modal-content">
            <div class="modal-header">
                <h4 class="modal-title text-danger" id="exception-title"><strong>Modal title</strong></h4>
            </div>
            <div class="modal-body">
                <p id="exception-body">Your error message goes here...</p>
            </div>
            <div class="modal-footer">
                <button type="button" class="btn btn-default" onclick='$("#exception").hide();'>Ignore</button>
                <button type="button" class="btn btn-danger" onclick='clearException();'>Clear Exception</button>
            </div>
        </div>
  </div>
</div>
