{% if playerArray %}
<div id="playerpanel" class="panel panel-primary">
    <div class="panel-heading">
        <div class="dropdown">
            <a class="dropbown-toggle panel-title" data-toggle="dropdown" href="#">{{activePlayer}}
                <span class="caret"></span>
            </a>
            <ul class="dropdown-menu">
              {% for player in playerArray%}
                <li>
                    <a href="./selectPlayer?id={{loop.index0}}">{{player}}</a>
                </li>
               {% endfor %}
            </ul>
        </div>
    </div>
    <div id="player-text" class="panel-body">
        {# note: disabled template because we no longer use this
            <div class="btn-group btn-group-justified">
            {% include "playerbutton.html" %}
            </div>
        #}
        {% if playerStatus == 0 %}
            Player - {{activePlayer}} - stopped
        {% elif playerStatus == 1 %}
            Player - {{activePlayer}} - playing {{ currentFile }}
        {% elif playerStatus == 2 %}
            Player - {{activePlayer}} - paused {{ currentFile }}
        {% endif %}
    </div>
</div>
{% endif %}
